<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    <title>{{ title }}</title>
    <link rel="icon" href="/static/favicon.svg" type="image/svg+xml" />
    <link rel="stylesheet" href="/static/console.css" />
  </head>
  <body>
    {% if deployment.name and not deployment.is_production %}
    <div class="deployment-banner" style="background: {{ deployment.color }};">{{ deployment.name|upper }}</div>
    {% endif %}
    <main class="shell">
      <div class="page" style="max-width: 720px; padding-top: 10vh;">
        <section class="panel" style="padding: 2rem;">
          <div class="eyebrow">Dewey Console</div>
          <h1 class="title" style="font-size: 2.3rem; margin-top: 0.45rem;">Dewey Access Login</h1>
          <p class="panel-copy" style="font-size: 1rem; max-width: 52ch;">
            Canonical artifact intake, lifecycle, sharing, and search for Dewey users and admins.
            Authentication is handled through Cognito Hosted UI.
          </p>
          <div class="form-actions" style="margin-top: 1.5rem;">
            <a class="button button-primary" href="{{ cognito_login_url }}">Sign In with Cognito</a>
            <span class="pill">Control Room UI</span>
          </div>
        </section>
      </div>
    </main>
  </body>
</html>
